@@ -241,11 +241,13 @@ static int nv_drm_connector_get_modes(struct drm_connector *connector)
         struct drm_display_mode *mode;
         struct NvKmsKapiDisplayMode displayMode;
         NvBool valid = 0;
+        NvBool preferredMode = NV_FALSE;
         int ret;
 
         ret = nvKms->getDisplayMode(nv_dev->pDevice,
                                     nv_detected_encoder->hDisplay,
-                                    modeIndex++, &displayMode, &valid);
+                                    modeIndex++, &displayMode, &valid,
+                                    &preferredMode);
 
         if (ret < 0) {
             NV_DRM_DEV_LOG_ERR(
@@ -279,6 +281,10 @@ static int nv_drm_connector_get_modes(struct drm_connector *connector)
 
         nvkms_display_mode_to_drm_mode(&displayMode, mode);
 
+        if (preferredMode) {
+            mode->type |= DRM_MODE_TYPE_PREFERRED;
+        }
+
         /* Add a mode to a connector's probed_mode list */
 
         drm_mode_probed_add(connector, mode);
