@@ -38,20 +38,23 @@
 #include "uvm8_gpu_non_replayable_faults.h"
 #include "uvm8_ats_ibm.h"
 #include "uvm8_ats_faults.h"
+#include "uvm8_test.h"
 
 // TODO: Bug 1881601: [uvm] Add fault handling overview for replayable and
 // non-replayable faults
 
 #define UVM_PERF_REENABLE_PREFETCH_FAULTS_LAPSE_MSEC_DEFAULT 1000
 
-// Lapse of time in milliseconds after which prefetch faults can be re-enabled. 0 means it is never disabled
+// Lapse of time in milliseconds after which prefetch faults can be re-enabled.
+// 0 means it is never disabled
 static unsigned uvm_perf_reenable_prefetch_faults_lapse_msec = UVM_PERF_REENABLE_PREFETCH_FAULTS_LAPSE_MSEC_DEFAULT;
 module_param(uvm_perf_reenable_prefetch_faults_lapse_msec, uint, S_IRUGO);
 
 #define UVM_PERF_FAULT_BATCH_COUNT_MIN 1
 #define UVM_PERF_FAULT_BATCH_COUNT_DEFAULT 256
 
-// Number of entries that are fetched from the GPU fault buffer and serviced in batch
+// Number of entries that are fetched from the GPU fault buffer and serviced in
+// batch
 static unsigned uvm_perf_fault_batch_count = UVM_PERF_FAULT_BATCH_COUNT_DEFAULT;
 module_param(uvm_perf_fault_batch_count, uint, S_IRUGO);
 
@@ -163,7 +166,7 @@ static NV_STATUS fault_buffer_init_replayable_faults(uvm_gpu_t *gpu)
 
     batch_context->max_utlb_id = 0;
 
-    status = uvm_rm_locked_call(nvUvmInterfaceOwnPageFaultIntr(&gpu->uuid, NV_TRUE));
+    status = uvm_rm_locked_call(nvUvmInterfaceOwnPageFaultIntr(gpu->rm_device, NV_TRUE));
     if (status != NV_OK) {
         UVM_ERR_PRINT("Failed to take page fault ownership from RM: %s, GPU %s\n",
                       nvstatusToString(status), gpu->name);
@@ -226,7 +229,7 @@ NV_STATUS uvm_gpu_fault_buffer_init(uvm_gpu_t *gpu)
     uvm_assert_mutex_locked(&g_uvm_global.global_lock);
     UVM_ASSERT(gpu->fault_buffer_hal != NULL);
 
-    status = uvm_rm_locked_call(nvUvmInterfaceInitFaultInfo(gpu->rm_address_space,
+    status = uvm_rm_locked_call(nvUvmInterfaceInitFaultInfo(gpu->rm_device,
                                                             &gpu->fault_buffer_info.rm_info));
     if (status != NV_OK) {
         UVM_ERR_PRINT("Failed to init fault buffer info from RM: %s, GPU %s\n",
@@ -279,10 +282,10 @@ void uvm_gpu_fault_buffer_deinit(uvm_gpu_t *gpu)
     fault_buffer_deinit_replayable_faults(gpu);
 
     if (gpu->fault_buffer_info.rm_info.faultBufferHandle) {
-        status = uvm_rm_locked_call(nvUvmInterfaceOwnPageFaultIntr(&gpu->uuid, NV_FALSE));
+        status = uvm_rm_locked_call(nvUvmInterfaceOwnPageFaultIntr(gpu->rm_device, NV_FALSE));
         UVM_ASSERT(status == NV_OK);
 
-        uvm_rm_locked_call_void(nvUvmInterfaceDestroyFaultInfo(gpu->rm_address_space,
+        uvm_rm_locked_call_void(nvUvmInterfaceDestroyFaultInfo(gpu->rm_device,
                                                                &gpu->fault_buffer_info.rm_info));
 
         gpu->fault_buffer_info.rm_info.faultBufferHandle = 0;
@@ -300,9 +303,11 @@ bool uvm_gpu_replayable_faults_pending(uvm_gpu_t *gpu)
     if (replayable_faults->cached_get != replayable_faults->cached_put)
         return true;
 
-    // Fast path 2: read the valid bit of the fault buffer entry pointed by the cached get pointer
+    // Fast path 2: read the valid bit of the fault buffer entry pointed by the
+    // cached get pointer
     if (!gpu->fault_buffer_hal->entry_is_valid(gpu, replayable_faults->cached_get)) {
-        // Slow path: read the put pointer from the GPU register via BAR0 over PCIe
+        // Slow path: read the put pointer from the GPU register via BAR0
+        // over PCIe
         replayable_faults->cached_put = gpu->fault_buffer_hal->read_put(gpu);
 
         // No interrupt pending
@@ -313,8 +318,9 @@ bool uvm_gpu_replayable_faults_pending(uvm_gpu_t *gpu)
     return true;
 }
 
-// Push a fault cancel method on the given client. Any failure during this operation may lead to
-// application hang (requiring manual Ctrl+C from the user) or system crash (requiring reboot).
+// Push a fault cancel method on the given client. Any failure during this
+// operation may lead to application hang (requiring manual Ctrl+C from the
+// user) or system crash (requiring reboot).
 // In that case we log an error message.
 //
 // This function acquires both the given tracker and the replay tracker
@@ -346,7 +352,8 @@ static NV_STATUS push_cancel_on_gpu(uvm_gpu_t *gpu, uvm_gpu_phys_address_t insta
 
     gpu->host_hal->cancel_faults_targeted(&push, instance_ptr, gpc_id, client_id);
 
-    // We don't need to put the cancel in the GPU replay tracker since we wait on it immediately.
+    // We don't need to put the cancel in the GPU replay tracker since we wait
+    // on it immediately.
     status = uvm_push_end_and_wait(&push);
 
     UVM_ASSERT(status == NV_OK);
@@ -409,7 +416,8 @@ static NV_STATUS cancel_fault_precise_va(uvm_gpu_t *gpu,
     }
     fault_entry->fault_address = UVM_PAGE_ALIGN_DOWN(fault_entry->fault_address - 1);
 
-    // We don't need to put the cancel in the GPU replay tracker since we wait on it immediately.
+    // We don't need to put the cancel in the GPU replay tracker since we wait
+    // on it immediately.
     status = uvm_push_end_and_wait(&push);
     if (status != NV_OK) {
         UVM_ERR_PRINT("Failed to wait for pushed VA global fault cancel: %s, GPU %s\n",
@@ -439,8 +447,9 @@ static NV_STATUS push_replay_on_gpu(uvm_gpu_t *gpu, uvm_fault_replay_type_t type
     gpu->host_hal->replay_faults(&push, type);
 
     // Do not count REPLAY_TYPE_START_ACK_ALL's toward the replay count.
-    // REPLAY_TYPE_START_ACK_ALL's are issued for cancels, and the cancel algorithm checks to make sure that
-    // no REPLAY_TYPE_START's have been issued using batch_context->replays.
+    // REPLAY_TYPE_START_ACK_ALL's are issued for cancels, and the cancel
+    // algorithm checks to make sure that no REPLAY_TYPE_START's have been
+    // issued using batch_context->replays.
     if (batch_context && type != UVM_FAULT_REPLAY_TYPE_START_ACK_ALL) {
         uvm_tools_broadcast_replay(gpu, &push, batch_context->batch_id, UVM_FAULT_CLIENT_TYPE_GPC);
         ++batch_context->num_replays;
@@ -816,7 +825,8 @@ done:
     batch_context->num_coalesced_faults = num_coalesced_faults;
 }
 
-// Sort comparator for pointers to fault buffer entries that sorts by instance pointer
+// Sort comparator for pointers to fault buffer entries that sorts by
+// instance pointer
 static int cmp_sort_fault_entry_by_instance_ptr(const void *_a, const void *_b)
 {
     const uvm_fault_buffer_entry_t **a = (const uvm_fault_buffer_entry_t **)_a;
@@ -825,7 +835,8 @@ static int cmp_sort_fault_entry_by_instance_ptr(const void *_a, const void *_b)
     return cmp_fault_instance_ptr(*a, *b);
 }
 
-// Sort comparator for pointers to fault buffer entries that sorts by va_space, fault address and fault access type
+// Sort comparator for pointers to fault buffer entries that sorts by va_space,
+// fault address and fault access type
 static int cmp_sort_fault_entry_by_va_space_address_access_type(const void *_a, const void *_b)
 {
     const uvm_fault_buffer_entry_t **a = (const uvm_fault_buffer_entry_t **)_a;
@@ -844,11 +855,12 @@ static int cmp_sort_fault_entry_by_va_space_address_access_type(const void *_a,
     return cmp_access_type((*a)->fault_access_type, (*b)->fault_access_type);
 }
 
-// Translate all instance pointers to VA spaces. Since the buffer is ordered by instance_ptr, we minimize the number of
-// translations
+// Translate all instance pointers to VA spaces. Since the buffer is ordered by
+// instance_ptr, we minimize the number of translations
 //
-// This function returns NV_WARN_MORE_PROCESSING_REQUIRED if a fault buffer flush occurred and
-// executed successfully, or the error code if it failed. NV_OK otherwise.
+// This function returns NV_WARN_MORE_PROCESSING_REQUIRED if a fault buffer
+// flush occurred and executed successfully, or the error code if it failed.
+// NV_OK otherwise.
 static NV_STATUS translate_instance_ptrs(uvm_gpu_t *gpu,
                                          uvm_fault_service_batch_context_t *batch_context)
 {
@@ -881,7 +893,7 @@ static NV_STATUS translate_instance_ptrs(uvm_gpu_t *gpu,
                 // context (TSG) is still running so we don't cancel an innocent
                 // context.
                 UVM_ASSERT(!current_entry->va_space);
-                UVM_ASSERT(gpu->max_subcontexts > 0);
+                UVM_ASSERT(gpu->rm_info.maxSubctxCount > 0);
 
                 status = push_cancel_on_gpu(gpu,
                                             current_entry->instance_ptr,
@@ -942,8 +954,8 @@ static NV_STATUS preprocess_fault_batch(uvm_gpu_t *gpu, uvm_fault_service_batch_
     UVM_ASSERT(batch_context->num_coalesced_faults > 0);
     UVM_ASSERT(batch_context->num_cached_faults >= batch_context->num_coalesced_faults);
 
-    // Generate an ordered view of the fault cache in ordered_fault_cache. We sort the pointers, not the entries
-    // in fault_cache
+    // Generate an ordered view of the fault cache in ordered_fault_cache.
+    // We sort the pointers, not the entries in fault_cache
 
     // Initialize pointers before they are sorted. We only sort one instance per
     // coalesced fault
@@ -967,7 +979,8 @@ static NV_STATUS preprocess_fault_batch(uvm_gpu_t *gpu, uvm_fault_service_batch_
     if (status != NV_OK)
         return status;
 
-    // 3) sort by va_space, fault address (GPU already reports 4K-aligned address) and access type
+    // 3) sort by va_space, fault address (GPU already reports 4K-aligned
+    // address) and access type
     sort(ordered_fault_cache,
          batch_context->num_coalesced_faults,
          sizeof(*ordered_fault_cache),
@@ -1213,9 +1226,9 @@ static NV_STATUS service_batch_managed_faults_in_block_locked(uvm_gpu_t *gpu,
                                                       &read_duplicate);
 
         if (!uvm_processor_mask_test_and_set(&block_context->resident_processors, new_residency))
-            uvm_page_mask_zero(&block_context->per_processor_masks[new_residency].new_residency);
+            uvm_page_mask_zero(&block_context->per_processor_masks[uvm_id_value(new_residency)].new_residency);
 
-        uvm_page_mask_set(&block_context->per_processor_masks[new_residency].new_residency, page_index);
+        uvm_page_mask_set(&block_context->per_processor_masks[uvm_id_value(new_residency)].new_residency, page_index);
 
         if (read_duplicate) {
             if (block_context->read_duplicate_count++ == 0)
@@ -1320,7 +1333,8 @@ typedef enum
     // Use this mode when calling from the normal fault servicing path
     FAULT_SERVICE_MODE_REGULAR,
 
-    // Use this mode when servicing faults from the fault cancelling algorithm. In this mode no replays are issued
+    // Use this mode when servicing faults from the fault cancelling algorithm.
+    // In this mode no replays are issued
     FAULT_SERVICE_MODE_CANCEL,
 } fault_service_mode_t;
 
@@ -1343,7 +1357,8 @@ static NV_STATUS service_non_managed_fault(uvm_fault_buffer_entry_t *current_ent
                        (current_entry->fault_address == previous_entry->fault_address);
 
         if (is_duplicate) {
-            // Propagate the is_invalid_prefetch flag across all prefetch faults on the page
+            // Propagate the is_invalid_prefetch flag across all prefetch faults
+            // on the page
             if (previous_entry->is_invalid_prefetch)
                 current_entry->is_invalid_prefetch = true;
 
@@ -1491,7 +1506,8 @@ static NV_STATUS service_fault_batch(uvm_gpu_t *gpu,
             // VA space is handled next
         }
 
-        // Some faults could be already fatal if they cannot be handled by the UVM driver
+        // Some faults could be already fatal if they cannot be handled by
+        // the UVM driver
         if (current_entry->is_fatal) {
             ++i;
             batch_context->has_fatal_faults = true;
@@ -1629,7 +1645,8 @@ static void faults_for_page_in_utlb(uvm_fault_service_batch_context_t *batch_con
     }
 }
 
-// Function that tells if there are addresses (reminder: they are aligned to 4K) with non-fatal faults only
+// Function that tells if there are addresses (reminder: they are aligned to 4K)
+// with non-fatal faults only
 static bool no_fatal_pages_in_utlb(uvm_fault_service_batch_context_t *batch_context,
                                    NvU32 start_index,
                                    NvU32 utlb_id)
@@ -1674,16 +1691,18 @@ static void record_fatal_fault_helper(uvm_gpu_t *gpu, uvm_fault_buffer_entry_t *
     uvm_va_space_up_read(va_space);
 }
 
-// This function tries to find and issue a cancel for each uTLB that meets the requirements to guarantee precise
-// fault attribution:
+// This function tries to find and issue a cancel for each uTLB that meets
+// the requirements to guarantee precise fault attribution:
 // - No new faults can arrive on the uTLB (uTLB is in lockdown)
 // - The first fault in the buffer for a specific uTLB is fatal
 // - There are no other addresses in the uTLB with non-fatal faults only
 //
-// This function and the related helpers iterate over faults as read from HW, not through the ordered fault view
+// This function and the related helpers iterate over faults as read from HW,
+// not through the ordered fault view
 //
 // TODO: Bug 1766754
-// This is very costly, although not critical for performance since we are cancelling.
+// This is very costly, although not critical for performance since we are
+// cancelling.
 // - Build a list with all the faults within a uTLB
 // - Sort by uTLB id
 static NV_STATUS try_to_cancel_utlbs(uvm_gpu_t *gpu, uvm_fault_service_batch_context_t *batch_context)
@@ -1812,17 +1831,21 @@ static NV_STATUS cancel_faults_precise_va(uvm_gpu_t *gpu,
             // ... and take the lock of the new one
             uvm_va_space_down_read(va_space);
 
-            // We don't need to check whether a buffer flush is required (due to VA range destruction).
-            // - For cancel_mode == FAULT_CANCEL_MODE_FATAL, once a fault is flagged as fatal we need to cancel it,
-            // even if its VA range no longer exists.
-            // - For cancel_mode == FAULT_CANCEL_MODE_ALL we don't care about any of this, we just want to trigger
-            // RC in RM.
+            // We don't need to check whether a buffer flush is required
+            // (due to VA range destruction).
+            // - For cancel_mode == FAULT_CANCEL_MODE_FATAL, once a fault is
+            // flagged as fatal we need to cancel it, even if its VA range no
+            // longer exists.
+            // - For cancel_mode == FAULT_CANCEL_MODE_ALL we don't care about
+            // any of this, we just want to trigger RC in RM.
         }
 
         if (!uvm_processor_mask_test(&va_space->registered_gpu_va_spaces, gpu->id)) {
-            // If there is no GPU VA space for the GPU, ignore the fault. This can happen if the GPU VA did not exist
-            // in service_fault_batch, or it was destroyed since then. This is to avoid targetting a PDB that might
-            // have been reused by another process.
+            // If there is no GPU VA space for the GPU, ignore the fault.
+            // This can happen if the GPU VA did not exist in
+            // service_fault_batch(), or it was destroyed since then.
+            // This is to avoid targetting a PDB that might have been reused
+            // by another process.
             continue;
         }
 
@@ -1830,8 +1853,8 @@ static NV_STATUS cancel_faults_precise_va(uvm_gpu_t *gpu,
         if (cancel_mode == FAULT_CANCEL_MODE_ALL || current_entry->is_fatal) {
             uvm_fault_cancel_va_mode_t cancel_va_mode = current_entry->replayable.cancel_va_mode;
 
-            // If cancelling unconditionally and the fault was not fatal, set the cancel reason passed to this
-            // function
+            // If cancelling unconditionally and the fault was not fatal,
+            // set the cancel reason passed to this function
             if (!current_entry->is_fatal) {
                 current_entry->fatal_reason = reason;
                 cancel_va_mode = UVM_FAULT_CANCEL_VA_MODE_ALL;
@@ -1849,7 +1872,8 @@ static NV_STATUS cancel_faults_precise_va(uvm_gpu_t *gpu,
     return status;
 }
 
-// Function called when the system has fould a global error and needs to trigger RC in RM
+// Function called when the system has found a global error and needs to
+// trigger RC in RM.
 // We cancel one entry per uTLB
 static void cancel_fault_batch_tlb(uvm_gpu_t *gpu,
                                    uvm_fault_service_batch_context_t *batch_context,
@@ -1901,23 +1925,27 @@ static void cancel_fault_batch(uvm_gpu_t *gpu,
 
 // Current fault cancel algorithm
 //
-// 1- Disable prefetching to avoid new requests keep coming and flooding the buffer
+// 1- Disable prefetching to avoid new requests keep coming and flooding the
+// buffer.
 // LOOP
 //   2- Record one fatal fault per uTLB to check if it shows up after the replay
-//   3- Flush fault buffer (REPLAY_TYPE_START_ACK_ALL to prevent new faults from coming to TLBs with pending faults)
+//   3- Flush fault buffer (REPLAY_TYPE_START_ACK_ALL to prevent new faults from
+//      coming to TLBs with pending faults)
 //   4- Wait for replay to finish
 //   5- Fetch all faults from buffer
 //   6- Check what uTLBs are in lockdown mode and can be cancelled
 //   7- Preprocess faults (order per va_space, fault address, access type)
 //   8- Service all non-fatal faults and mark all non-serviceable faults as fatal
 //      6.1- If fatal faults are not found, we are done
-//   9- Search for a uTLB which can be targeted for cancel, as described in try_to_cancel_utlbs. If found, cancel it.
+//   9- Search for a uTLB which can be targeted for cancel, as described in
+//      try_to_cancel_utlbs. If found, cancel it.
 // END LOOP
 // 10- Re-enable prefetching
 //
-// NOTE: prefetch faults MUST NOT trigger fault cancel. We make sure that no prefetch faults are left in the buffer
-// by disabling prefetching and flushing the fault buffer afterwards (prefetch faults are not replayed and, therefore,
-// will not show up again)
+// NOTE: prefetch faults MUST NOT trigger fault cancel. We make sure that no
+// prefetch faults are left in the buffer by disabling prefetching and
+// flushing the fault buffer afterwards (prefetch faults are not replayed and,
+// therefore, will not show up again)
 static NV_STATUS cancel_faults_precise_tlb(uvm_gpu_t *gpu, uvm_fault_service_batch_context_t *batch_context)
 {
     NV_STATUS status;
@@ -1927,15 +1955,18 @@ static NV_STATUS cancel_faults_precise_tlb(uvm_gpu_t *gpu, uvm_fault_service_bat
 
     UVM_ASSERT(gpu->replayable_faults_supported);
 
-    // 1) Disable prefetching to avoid new requests keep coming and flooding the buffer
+    // 1) Disable prefetching to avoid new requests keep coming and flooding
+    //    the buffer
     if (gpu->fault_buffer_info.prefetch_faults_enabled)
         gpu->arch_hal->disable_prefetch_faults(gpu);
 
     while (1) {
         NvU32 utlb_id;
 
-        // 2) Record one fatal fault per uTLB to check if it shows up after the replay. This is used to handle the
-        // case in which the uTLB is being cancelled from behind our backs by RM. See the comment in step 6.
+        // 2) Record one fatal fault per uTLB to check if it shows up after
+        // the replay. This is used to handle the case in which the uTLB is
+        // being cancelled from behind our backs by RM. See the comment in
+        // step 6.
         for (utlb_id = 0; utlb_id <= batch_context->max_utlb_id; ++utlb_id) {
             uvm_fault_utlb_info_t *utlb = &batch_context->utlbs[utlb_id];
 
@@ -1951,10 +1982,12 @@ static NV_STATUS cancel_faults_precise_tlb(uvm_gpu_t *gpu, uvm_fault_service_bat
         }
         first = false;
 
-        // 3) Flush fault buffer. After this call, all faults from any of the faulting uTLBs are before PUT. New
-        // faults from other uTLBs can keep arriving. Therefore, in each iteration we just try to cancel faults
-        // from uTLBs that contained fatal faults in the previous iterations and will cause the TLB to stop
-        // generating new page faults after the following replay with type UVM_FAULT_REPLAY_TYPE_START_ACK_ALL
+        // 3) Flush fault buffer. After this call, all faults from any of the
+        // faulting uTLBs are before PUT. New faults from other uTLBs can keep
+        // arriving. Therefore, in each iteration we just try to cancel faults
+        // from uTLBs that contained fatal faults in the previous iterations
+        // and will cause the TLB to stop generating new page faults after the
+        // following replay with type UVM_FAULT_REPLAY_TYPE_START_ACK_ALL
         status = fault_buffer_flush_locked(gpu,
                                            UVM_GPU_BUFFER_FLUSH_MODE_UPDATE_PUT,
                                            UVM_FAULT_REPLAY_TYPE_START_ACK_ALL,
@@ -1990,11 +2023,14 @@ static NV_STATUS cancel_faults_precise_tlb(uvm_gpu_t *gpu, uvm_fault_service_bat
             utlb->cancelled   = false;
 
             if (utlb->prev_fatal_fault.fault_address != (NvU64)-1) {
-                // If a previously-reported fault shows up again we can "safely" assume that the uTLB that contains it
-                // is in lockdown mode and no new translations will show up before cancel. A fatal fault could only
-                // be removed behind our backs by RM issuing a cancel, which only happens when RM is resetting the
-                // engine. That means the instance pointer can't generate any new faults, so we won't have an ABA
-                // problem where a new fault arrives with the same state.
+                // If a previously-reported fault shows up again we can "safely"
+                // assume that the uTLB that contains it is in lockdown mode
+                // and no new translations will show up before cancel.
+                // A fatal fault could only be removed behind our backs by RM
+                // issuing a cancel, which only happens when RM is resetting the
+                // engine. That means the instance pointer can't generate any
+                // new faults, so we won't have an ABA problem where a new
+                // fault arrives with the same state.
                 if (is_fatal_fault_in_buffer(batch_context, &utlb->prev_fatal_fault))
                     utlb->in_lockdown = true;
             }
@@ -2007,7 +2043,8 @@ static NV_STATUS cancel_faults_precise_tlb(uvm_gpu_t *gpu, uvm_fault_service_bat
         else if (status != NV_OK)
             break;
 
-        // 8) Service all non-fatal faults and mark all non-serviceable faults as fatal
+        // 8) Service all non-fatal faults and mark all non-serviceable faults
+        // as fatal
         status = service_fault_batch(gpu, FAULT_SERVICE_MODE_CANCEL, batch_context);
         if (status == NV_WARN_MORE_PROCESSING_REQUIRED)
             continue;
@@ -2022,7 +2059,8 @@ static NV_STATUS cancel_faults_precise_tlb(uvm_gpu_t *gpu, uvm_fault_service_bat
         if (!batch_context->has_fatal_faults)
             break;
 
-        // 9) Search for uTLBs that contain fatal faults and meet the requirements to be cancelled
+        // 9) Search for uTLBs that contain fatal faults and meet the
+        // requirements to be cancelled
         try_to_cancel_utlbs(gpu, batch_context);
     }
 
@@ -2056,7 +2094,8 @@ static void enable_disable_prefetch_faults(uvm_gpu_t *gpu, uvm_fault_service_bat
     if (!gpu->prefetch_fault_supported)
         return;
 
-    // If more than 66% of faults are invalid prefetch accesses, disable prefetch faults for a while
+    // If more than 66% of faults are invalid prefetch accesses, disable
+    // prefetch faults for a while
     if (gpu->fault_buffer_info.prefetch_faults_enabled &&
         ((batch_context->num_invalid_prefetch_faults * 3 > gpu->fault_buffer_info.max_batch_size * 2 &&
           uvm_perf_reenable_prefetch_faults_lapse_msec > 0) ||
@@ -2125,10 +2164,12 @@ void uvm_gpu_service_replayable_faults(uvm_gpu_t *gpu)
         enable_disable_prefetch_faults(gpu, batch_context);
 
         if (status != NV_OK) {
-            // Unconditionally cancel all faults to trigger RC. This will not provide precise
-            // attribution, but this case handles global errors such as OOM or ECC where it's
-            // not reasonable to guarantee precise attribution. We ignore the return value
-            // of the cancel operation since this path is already returning an error code.
+            // Unconditionally cancel all faults to trigger RC. This will not
+            // provide precise attribution, but this case handles global
+            // errors such as OOM or ECC where it's not reasonable to
+            // guarantee precise attribution. We ignore the return value of
+            // the cancel operation since this path is already returning an
+            // error code.
             cancel_fault_batch(gpu, batch_context, uvm_tools_status_to_fatal_fault_reason(status));
             break;
         }
@@ -2176,8 +2217,8 @@ void uvm_gpu_service_replayable_faults(uvm_gpu_t *gpu)
     if (status == NV_WARN_MORE_PROCESSING_REQUIRED)
         status = NV_OK;
 
-    // Make sure that we issue at least one replay if no replay has been issued yet to avoid dropping faults that do
-    // not show up in the buffer
+    // Make sure that we issue at least one replay if no replay has been
+    // issued yet to avoid dropping faults that do not show up in the buffer
     if ((status == NV_OK && replayable_faults->replay_policy == UVM_PERF_FAULT_REPLAY_POLICY_ONCE) ||
         num_replays == 0)
         status = push_replay_on_gpu(gpu, UVM_FAULT_REPLAY_TYPE_START, batch_context);
