@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2018 NVIDIA Corporation
+    Copyright (c) 2016-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -21,6 +21,7 @@
 
 *******************************************************************************/
 
+#include "uvm8_api.h"
 #include "uvm8_perf_events.h"
 #include "uvm8_perf_module.h"
 #include "uvm8_perf_thrashing.h"
@@ -30,6 +31,7 @@
 #include "uvm8_kvmalloc.h"
 #include "uvm8_tools.h"
 #include "uvm8_procfs.h"
+#include "uvm8_test.h"
 
 // Number of bits for page-granularity time stamps. Currently we ignore the first 6 bits
 // of the timestamp (i.e. we have 64ns resolution, which is good enough)
@@ -78,10 +80,10 @@ typedef struct
     struct
     {
         // Memory residency for the page when in pinning phase
-        uvm_processor_id_t          pinned_residency : 16;
+        NvU32                   pinned_residency_idx : 16;
 
         // Processor not to be throttled in the current throttling period
-        uvm_processor_id_t do_not_throttle_processor : 16;
+        NvU32          do_not_throttle_processor_idx : 16;
     };
 } page_thrashing_info_t;
 
@@ -173,8 +175,16 @@ typedef struct
 
     struct
     {
+        // Whether thrashing mitigation is enabled on this VA space
         bool                                  enable;
 
+        // true if the thrashing mitigation parameters have been modified using
+        // test ioctls
+        bool                          test_overrides;
+
+        //
+        // Fields below are the thrashing mitigation parameters on the VA space
+        //
         unsigned                           threshold;
 
         unsigned                       pin_threshold;
@@ -215,11 +225,11 @@ typedef struct
 // uvm_procfs_is_debug_enabled() returns true.
 static processor_thrashing_stats_t g_cpu_thrashing_stats;
 
-#define PROCESSOR_THRASHING_STATS_INC(proc, field)                                         \
-    do {                                                                                   \
-        processor_thrashing_stats_t *_processor_stats = thrashing_stats_get_or_null(proc); \
-        if (_processor_stats)                                                              \
-            atomic64_inc(&_processor_stats->field);                                        \
+#define PROCESSOR_THRASHING_STATS_INC(va_space, proc, field)                                         \
+    do {                                                                                             \
+        processor_thrashing_stats_t *_processor_stats = thrashing_stats_get_or_null(va_space, proc); \
+        if (_processor_stats)                                                                        \
+            atomic64_inc(&_processor_stats->field);                                                  \
     } while (0)
 
 // Global caches for the per-VA block thrashing detection structures
@@ -253,20 +263,21 @@ static unsigned uvm_perf_thrashing_pin_threshold = UVM_PERF_THRASHING_PIN_THRESH
 
 // TODO: Bug 1768615: [uvm8] Automatically tune default values for thrashing
 // detection/prevention parameters
-#define UVM_PERF_THRASHING_LAPSE_USEC_DEFAULT 250
+#define UVM_PERF_THRASHING_LAPSE_USEC_DEFAULT 500
+#define UVM_PERF_THRASHING_LAPSE_USEC_DEFAULT_EMULATION (UVM_PERF_THRASHING_LAPSE_USEC_DEFAULT * 800)
 
 // Lapse of time in microseconds that determines if two consecutive events on
 // the same page can be considered thrashing
 static unsigned uvm_perf_thrashing_lapse_usec = UVM_PERF_THRASHING_LAPSE_USEC_DEFAULT;
 
-#define UVM_PERF_THRASHING_NAP_DEFAULT 2
-#define UVM_PERF_THRASHING_NAP_MAX     1000
+#define UVM_PERF_THRASHING_NAP_DEFAULT 1
+#define UVM_PERF_THRASHING_NAP_MAX     100
 
 // Time that the processor being throttled is forbidden to work on the thrashing
 // page. This value is a multiplier of uvm_perf_thrashing_lapse_usec.
 static unsigned uvm_perf_thrashing_nap = UVM_PERF_THRASHING_NAP_DEFAULT;
 
-#define UVM_PERF_THRASHING_EPOCH_DEFAULT 4000
+#define UVM_PERF_THRASHING_EPOCH_DEFAULT 2000
 
 // Time lapse after which we consider thrashing is no longer happening. This
 // value is a multiplier of uvm_perf_thrashing_lapse_usec.
@@ -280,7 +291,8 @@ static unsigned uvm_perf_thrashing_epoch = UVM_PERF_THRASHING_EPOCH_DEFAULT;
 // some time. We use a per-VA space list of pinned pages, sorted by the
 // deadline at which it will be unmapped from remote processors. Therefore,
 // next remote access will trigger a fault that will migrate the page.
-#define UVM_PERF_THRASHING_PIN_DEFAULT 600
+#define UVM_PERF_THRASHING_PIN_DEFAULT 300
+#define UVM_PERF_THRASHING_PIN_DEFAULT_EMULATION 10
 
 // Time for which a page remains pinned. This value is a multiplier of
 // uvm_perf_thrashing_lapse_usec. 0 means that it is pinned forever.
@@ -412,7 +424,12 @@ static int nv_procfs_read_thrashing_stats(struct seq_file *s, void *v)
     return 0;
 }
 
-UVM_DEFINE_SINGLE_PROCFS_FILE(thrashing_stats);
+static int nv_procfs_read_thrashing_stats_entry(struct seq_file *s, void *v)
+{
+    UVM_ENTRY_RET(nv_procfs_read_thrashing_stats(s, v));
+}
+
+UVM_DEFINE_SINGLE_PROCFS_FILE(thrashing_stats_entry);
 
 #define THRASHING_STATS_FILE_NAME "thrashing_stats"
 
@@ -426,7 +443,7 @@ static NV_STATUS cpu_thrashing_stats_init(void)
         UVM_ASSERT(!g_cpu_thrashing_stats.procfs_file);
         g_cpu_thrashing_stats.procfs_file = NV_CREATE_PROC_FILE(THRASHING_STATS_FILE_NAME,
                                                                 cpu_base_dir_entry,
-                                                                thrashing_stats,
+                                                                thrashing_stats_entry,
                                                                 &g_cpu_thrashing_stats);
         if (!g_cpu_thrashing_stats.procfs_file)
             return NV_ERR_OPERATING_SYSTEM;
@@ -452,16 +469,16 @@ static processor_thrashing_stats_t *gpu_thrashing_stats_get_or_null(uvm_gpu_t *g
     return uvm_perf_module_type_data(gpu->perf_modules_data, UVM_PERF_MODULE_TYPE_THRASHING);
 }
 
-static processor_thrashing_stats_t *thrashing_stats_get_or_null(uvm_processor_id_t id)
+static processor_thrashing_stats_t *thrashing_stats_get_or_null(uvm_va_space_t *va_space, uvm_processor_id_t id)
 {
-    if (id == UVM_CPU_ID) {
+    if (UVM_ID_IS_CPU(id)) {
         if (g_cpu_thrashing_stats.procfs_file)
             return &g_cpu_thrashing_stats;
 
         return NULL;
     }
 
-    return gpu_thrashing_stats_get_or_null(uvm_gpu_get(id));
+    return gpu_thrashing_stats_get_or_null(uvm_va_space_get_gpu(va_space, id));
 }
 
 // Create the thrashing stats struct for the given GPU
@@ -481,7 +498,7 @@ static NV_STATUS gpu_thrashing_stats_create(uvm_gpu_t *gpu)
 
     gpu_thrashing->procfs_file = NV_CREATE_PROC_FILE(THRASHING_STATS_FILE_NAME,
                                                      gpu->procfs.dir,
-                                                     thrashing_stats,
+                                                     thrashing_stats_entry,
                                                      gpu_thrashing);
     if (!gpu_thrashing->procfs_file) {
         uvm_kvfree(gpu_thrashing);
@@ -531,6 +548,39 @@ static va_space_thrashing_info_t *va_space_thrashing_info_get(uvm_va_space_t *va
     return va_space_thrashing;
 }
 
+static void va_space_thrashing_info_init_params(va_space_thrashing_info_t *va_space_thrashing)
+{
+    UVM_ASSERT(!va_space_thrashing->params.test_overrides);
+
+    va_space_thrashing->params.enable = g_uvm_perf_thrashing_enable;
+
+    // Snap the thrashing parameters so that they can be tuned per VA space
+    va_space_thrashing->params.threshold     = g_uvm_perf_thrashing_threshold;
+    va_space_thrashing->params.pin_threshold = g_uvm_perf_thrashing_pin_threshold;
+
+    // Default thrashing parameters are overriden for simulated/emulated GPUs
+    if (g_uvm_global.num_simulated_devices > 0 &&
+        (g_uvm_perf_thrashing_lapse_usec == UVM_PERF_THRASHING_LAPSE_USEC_DEFAULT)) {
+        va_space_thrashing->params.lapse_ns  = UVM_PERF_THRASHING_LAPSE_USEC_DEFAULT_EMULATION * 1000;
+    }
+    else {
+        va_space_thrashing->params.lapse_ns  = g_uvm_perf_thrashing_lapse_usec * 1000;
+    }
+
+    va_space_thrashing->params.nap_ns        = va_space_thrashing->params.lapse_ns * g_uvm_perf_thrashing_nap;
+    va_space_thrashing->params.epoch_ns      = va_space_thrashing->params.lapse_ns * g_uvm_perf_thrashing_epoch;
+
+    if (g_uvm_global.num_simulated_devices > 0 && (g_uvm_perf_thrashing_pin == UVM_PERF_THRASHING_PIN_DEFAULT)) {
+        va_space_thrashing->params.pin_ns    = va_space_thrashing->params.lapse_ns
+                                               * UVM_PERF_THRASHING_PIN_DEFAULT_EMULATION;
+    }
+    else {
+        va_space_thrashing->params.pin_ns    = va_space_thrashing->params.lapse_ns * g_uvm_perf_thrashing_pin;
+    }
+
+    va_space_thrashing->params.max_resets    = g_uvm_perf_thrashing_max_resets;
+}
+
 // Create the thrashing detection struct for the given VA space
 //
 // VA space lock needs to be held in write mode
@@ -543,18 +593,10 @@ static va_space_thrashing_info_t *va_space_thrashing_info_create(uvm_va_space_t
 
     va_space_thrashing = uvm_kvmalloc_zero(sizeof(*va_space_thrashing));
     if (va_space_thrashing) {
-        // Snap the thrashing parameters so that they can be tuned per VA space
-        va_space_thrashing->params.enable         = g_uvm_perf_thrashing_enable;
-        va_space_thrashing->params.threshold      = g_uvm_perf_thrashing_threshold;
-        va_space_thrashing->params.pin_threshold  = g_uvm_perf_thrashing_pin_threshold;
-        va_space_thrashing->params.lapse_ns       = g_uvm_perf_thrashing_lapse_usec * 1000;
-        va_space_thrashing->params.nap_ns         = va_space_thrashing->params.lapse_ns * g_uvm_perf_thrashing_nap;
-        va_space_thrashing->params.epoch_ns       = va_space_thrashing->params.lapse_ns * g_uvm_perf_thrashing_epoch;
-        va_space_thrashing->params.pin_ns         = va_space_thrashing->params.lapse_ns * g_uvm_perf_thrashing_pin;
-        va_space_thrashing->params.max_resets     = g_uvm_perf_thrashing_max_resets;
-
         va_space_thrashing->va_space = va_space;
 
+        va_space_thrashing_info_init_params(va_space_thrashing);
+
         uvm_perf_module_type_set_data(va_space->perf_modules_data, va_space_thrashing, UVM_PERF_MODULE_TYPE_THRASHING);
     }
 
@@ -589,14 +631,14 @@ static block_thrashing_info_t *thrashing_info_get_create(uvm_va_block_t *va_bloc
     block_thrashing_info_t *block_thrashing = thrashing_info_get(va_block);
 
     BUILD_BUG_ON((1 << 8 * sizeof(block_thrashing->num_thrashing_pages)) < PAGES_PER_UVM_VA_BLOCK);
-    BUILD_BUG_ON((1 << 16) < UVM_MAX_PROCESSORS);
+    BUILD_BUG_ON((1 << 16) < UVM_ID_MAX_PROCESSORS);
 
     if (!block_thrashing) {
         block_thrashing = kmem_cache_zalloc(g_va_block_thrashing_info_cache, NV_UVM_GFP_FLAGS);
         if (!block_thrashing)
             goto done;
 
-        block_thrashing->last_processor = UVM_MAX_PROCESSORS;
+        block_thrashing->last_processor = UVM_ID_INVALID;
         INIT_LIST_HEAD(&block_thrashing->pinned_pages.list);
 
         uvm_perf_module_type_set_data(va_block->perf_modules_data, block_thrashing, UVM_PERF_MODULE_TYPE_THRASHING);
@@ -682,12 +724,12 @@ static bool thrashing_state_checks(uvm_va_block_t *va_block,
 
     if (page_thrashing->pinned) {
         UVM_ASSERT(uvm_page_mask_test(&block_thrashing->pinned_pages.mask, page_index));
-        UVM_ASSERT(page_thrashing->pinned_residency != UVM_MAX_PROCESSORS);
+        UVM_ASSERT(UVM_ID_IS_VALID(uvm_id(page_thrashing->pinned_residency_idx)));
         UVM_ASSERT(page_thrashing->throttling_count == 0);
     }
     else {
         UVM_ASSERT(!uvm_page_mask_test(&block_thrashing->pinned_pages.mask, page_index));
-        UVM_ASSERT(page_thrashing->pinned_residency == UVM_MAX_PROCESSORS);
+        UVM_ASSERT(UVM_ID_IS_INVALID(uvm_id(page_thrashing->pinned_residency_idx)));
 
         if (!uvm_processor_mask_empty(&page_thrashing->throttled_processors)) {
             UVM_ASSERT(page_thrashing->throttling_count > 0);
@@ -716,13 +758,13 @@ static void thrashing_throttle_update(va_space_thrashing_info_t *va_space_thrash
         page_thrashing_set_throttling_end_time_stamp(page_thrashing, throttling_end_time_stamp);
 
         // Avoid choosing the same processor in consecutive thrashing periods
-        if (page_thrashing->do_not_throttle_processor == processor)
-            page_thrashing->do_not_throttle_processor = UVM_MAX_PROCESSORS;
+        if (uvm_id_equal(uvm_id(page_thrashing->do_not_throttle_processor_idx), processor))
+            page_thrashing->do_not_throttle_processor_idx = uvm_id_value(UVM_ID_INVALID);
         else
-            page_thrashing->do_not_throttle_processor = processor;
+            page_thrashing->do_not_throttle_processor_idx = uvm_id_value(processor);
     }
-    else if (page_thrashing->do_not_throttle_processor == UVM_MAX_PROCESSORS) {
-        page_thrashing->do_not_throttle_processor = processor;
+    else if (UVM_ID_IS_INVALID(uvm_id(page_thrashing->do_not_throttle_processor_idx))) {
+        page_thrashing->do_not_throttle_processor_idx = uvm_id_value(processor);
     }
 }
 
@@ -748,13 +790,13 @@ static void thrashing_throttle_processor(uvm_va_block_t *va_block,
     va_space = va_block->va_range->va_space;
     UVM_ASSERT(va_space);
 
-    UVM_ASSERT(processor != page_thrashing->do_not_throttle_processor);
+    UVM_ASSERT(!uvm_id_equal(processor, uvm_id(page_thrashing->do_not_throttle_processor_idx)));
 
     if (!uvm_processor_mask_test_and_set(&page_thrashing->throttled_processors, processor)) {
         // CPU is throttled by sleeping. This is done in uvm_vm_fault so it
         // drops the VA block and VA space locks. Throttling start/end events
         // are recorded around the sleep calls.
-        if (processor != UVM_CPU_ID)
+        if (UVM_ID_IS_GPU(processor))
             uvm_tools_record_throttling_start(va_space, address, processor);
 
         if (!page_thrashing->pinned)
@@ -793,7 +835,7 @@ static void thrashing_throttle_end_processor(uvm_va_block_t *va_block,
 
     // See comment regarding throttling start/end events for CPU in
     // thrashing_throttle_processor
-    if (processor != UVM_CPU_ID)
+    if (UVM_ID_IS_GPU(processor))
         uvm_tools_record_throttling_end(va_space, address, processor);
 
     UVM_ASSERT(thrashing_state_checks(va_block, block_thrashing, page_thrashing, page_index));
@@ -897,7 +939,7 @@ static NV_STATUS thrashing_pin_page(va_space_thrashing_info_t *va_space_thrashin
         uvm_page_mask_set(&block_thrashing->pinned_pages.mask, page_index);
     }
 
-    page_thrashing->pinned_residency = residency;
+    page_thrashing->pinned_residency_idx = uvm_id_value(residency);
 
     UVM_ASSERT(thrashing_state_checks(va_block, block_thrashing, page_thrashing, page_index));
 
@@ -951,7 +993,7 @@ static void thrashing_unpin_page(va_space_thrashing_info_t *va_space_thrashing,
             kmem_cache_free(g_pinned_page_cache, pinned_page);
     }
 
-    page_thrashing->pinned_residency = UVM_MAX_PROCESSORS;
+    page_thrashing->pinned_residency_idx = uvm_id_value(UVM_ID_INVALID);
     page_thrashing->pinned = false;
     uvm_page_mask_clear(&block_thrashing->pinned_pages.mask, page_index);
 
@@ -976,7 +1018,7 @@ static void thrashing_detected(uvm_va_block_t *va_block,
     if (!uvm_page_mask_test_and_set(&block_thrashing->thrashing_pages, page_index))
         ++block_thrashing->num_thrashing_pages;
 
-    PROCESSOR_THRASHING_STATS_INC(processor_id, num_thrashing);
+    PROCESSOR_THRASHING_STATS_INC(va_space, processor_id, num_thrashing);
 
     UVM_ASSERT(thrashing_state_checks(va_block, block_thrashing, page_thrashing, page_index));
 }
@@ -1051,7 +1093,7 @@ static NV_STATUS unmap_remote_pinned_pages_from_processors(uvm_va_block_t *va_bl
     uvm_assert_mutex_locked(&va_block->lock);
 
     for_each_id_in_mask(processor_id, unmap_processors) {
-        UVM_ASSERT(processor_id == va_range->preferred_location ||
+        UVM_ASSERT(uvm_id_equal(processor_id, va_range->preferred_location) ||
                    !uvm_processor_mask_test(&va_range->accessed_by, processor_id));
 
         if (uvm_processor_mask_test(&va_block->resident, processor_id)) {
@@ -1165,8 +1207,11 @@ static bool migrating_wrong_pages(uvm_va_block_t *va_block,
 
     for_each_va_block_page_in_region(page_index, region) {
         page_thrashing_info_t *page_thrashing = &block_thrashing->pages[page_index];
-        UVM_ASSERT_MSG(!page_thrashing->pinned || proc_id == page_thrashing->pinned_residency,
-                       "Migrating to %u instead of %u\n", proc_id, page_thrashing->pinned_residency);
+        uvm_processor_id_t pinned_residency = uvm_id(page_thrashing->pinned_residency_idx);
+        UVM_ASSERT_MSG(!page_thrashing->pinned || uvm_id_equal(proc_id, pinned_residency),
+                       "Migrating to %u instead of %u\n",
+                       uvm_id_value(proc_id),
+                       page_thrashing->pinned_residency_idx);
         if (cause == UVM_MAKE_RESIDENT_CAUSE_PREFETCH)
             UVM_ASSERT(!uvm_page_mask_test(&block_thrashing->thrashing_pages, page_index));
     }
@@ -1195,8 +1240,11 @@ static bool is_migration_pinned_pages_update(uvm_va_block_t *va_block,
     ret = uvm_page_mask_region_full(&block_thrashing->pinned_pages.mask, region);
     if (ret) {
         uvm_page_index_t page_index;
-        for_each_va_block_page_in_region(page_index, region)
-            UVM_ASSERT(block_thrashing->pages[page_index].pinned_residency == event_data->migration.dst);
+        for_each_va_block_page_in_region(page_index, region) {
+            page_thrashing_info_t *page_thrashing = &block_thrashing->pages[page_index];
+            uvm_processor_id_t pinned_residency = uvm_id(page_thrashing->pinned_residency_idx);
+            UVM_ASSERT(uvm_id_equal(pinned_residency, event_data->migration.dst));
+        }
     }
 
     return ret;
@@ -1233,7 +1281,7 @@ void thrashing_event_cb(uvm_perf_event_t event_id, uvm_perf_event_data_t *event_
             return;
 
         // Do not perform checks during the first part of staging copies
-        if (event_data->migration.dst != event_data->migration.make_resident_context->dest_id)
+        if (!uvm_id_equal(event_data->migration.dst, event_data->migration.make_resident_context->dest_id))
             return;
 
         va_space = va_block->va_range->va_space;
@@ -1293,7 +1341,7 @@ void thrashing_event_cb(uvm_perf_event_t event_id, uvm_perf_event_data_t *event_
         NvU16 num_block_pages;
 
         if (block_thrashing->last_time_stamp == 0 ||
-            block_thrashing->last_processor == processor_id ||
+            uvm_id_equal(block_thrashing->last_processor, processor_id) ||
             time_stamp - block_thrashing->last_time_stamp > va_space_thrashing->params.lapse_ns) {
             goto done;
         }
@@ -1305,8 +1353,8 @@ void thrashing_event_cb(uvm_perf_event_t event_id, uvm_perf_event_data_t *event_
             goto done;
 
         for (page_index = 0; page_index < num_block_pages; ++page_index) {
-            block_thrashing->pages[page_index].pinned_residency = UVM_MAX_PROCESSORS;
-            block_thrashing->pages[page_index].do_not_throttle_processor = UVM_MAX_PROCESSORS;
+            block_thrashing->pages[page_index].pinned_residency_idx = uvm_id_value(UVM_ID_INVALID);
+            block_thrashing->pages[page_index].do_not_throttle_processor_idx = uvm_id_value(UVM_ID_INVALID);
         }
     }
 
@@ -1354,6 +1402,67 @@ done:
     block_thrashing->last_processor  = processor_id;
 }
 
+static bool thrashing_processors_can_access(uvm_va_space_t *va_space,
+                                            page_thrashing_info_t *page_thrashing,
+                                            uvm_processor_id_t to)
+{
+    if (UVM_ID_IS_INVALID(to))
+        return false;
+
+    return uvm_processor_mask_subset(&page_thrashing->processors,
+                                     &va_space->accessible_from[uvm_id_value(to)]);
+}
+
+static bool thrashing_processors_have_fast_access_to(uvm_va_space_t *va_space,
+                                                     page_thrashing_info_t *page_thrashing,
+                                                     uvm_processor_id_t to)
+{
+    uvm_processor_mask_t fast_to;
+
+    if (UVM_ID_IS_INVALID(to))
+        return false;
+
+    // Combine NVLINK and native atomics mask since we could have PCIe
+    // atomics in the future
+    uvm_processor_mask_and(&fast_to,
+                           &va_space->has_nvlink[uvm_id_value(to)],
+                           &va_space->has_native_atomics[uvm_id_value(to)]);
+    uvm_processor_mask_set(&fast_to, to);
+
+    return uvm_processor_mask_subset(&page_thrashing->processors, &fast_to);
+}
+
+static void thrashing_processors_common_locations(uvm_va_space_t *va_space,
+                                                  page_thrashing_info_t *page_thrashing,
+                                                  uvm_processor_mask_t *common_locations)
+{
+    bool is_first = true;
+    uvm_processor_id_t id;
+
+    // Find processors that can be accessed from all thrashing processors. For
+    // example: if A, B and C are thrashing, and A can access B and C can access
+    // B, too, B would be the common location.
+    uvm_processor_mask_zero(common_locations);
+
+    for_each_id_in_mask(id, &page_thrashing->processors) {
+        if (is_first)
+            uvm_processor_mask_copy(common_locations, &va_space->can_access[uvm_id_value(id)]);
+        else
+            uvm_processor_mask_and(common_locations, common_locations, &va_space->can_access[uvm_id_value(id)]);
+
+        is_first = false;
+    }
+}
+
+static bool preferred_location_is_thrashing(uvm_va_range_t *va_range,
+                                            page_thrashing_info_t *page_thrashing)
+{
+    if (UVM_ID_IS_INVALID(va_range->preferred_location))
+        return false;
+
+    return uvm_processor_mask_test(&page_thrashing->processors, va_range->preferred_location);
+}
+
 static uvm_perf_thrashing_hint_t get_hint_for_migration_thrashing(va_space_thrashing_info_t *va_space_thrashing,
                                                                   uvm_va_block_t *va_block,
                                                                   uvm_page_index_t page_index,
@@ -1364,86 +1473,113 @@ static uvm_perf_thrashing_hint_t get_hint_for_migration_thrashing(va_space_thras
     uvm_processor_id_t closest_resident_id;
     uvm_va_range_t *va_range = va_block->va_range;
     uvm_va_space_t *va_space = va_range->va_space;
+    uvm_processor_id_t do_not_throttle_processor = uvm_id(page_thrashing->do_not_throttle_processor_idx);
+    uvm_processor_id_t pinned_residency = uvm_id(page_thrashing->pinned_residency_idx);
 
     hint.type = UVM_PERF_THRASHING_HINT_TYPE_NONE;
 
-    closest_resident_id = uvm_va_block_page_get_closest_resident_in_mask(va_block,
-                                                                         page_index,
-                                                                         requester,
-                                                                         &page_thrashing->processors);
+    closest_resident_id = uvm_va_block_page_get_closest_resident(va_block, page_index, requester);
+    UVM_ASSERT(UVM_ID_IS_VALID(closest_resident_id));
 
-    // 1) If preferred_location is set, try to map to it (throttle if that's not possible)
-    // 2) If NVLINK connection with support for native atomics, map it
-    // 3) Else first throttle, then map (if processors do not have access,
-    //    migrate, if necessary, and map to sysmem).
-    if (va_range->preferred_location != UVM_MAX_PROCESSORS) {
-        if (uvm_processor_mask_test(&va_space->accessible_from[va_range->preferred_location], requester)) {
+    if (thrashing_processors_can_access(va_space, page_thrashing, va_range->preferred_location)) {
+        // The logic in uvm_va_block_select_residency chooses the preferred
+        // location if the requester can access it, so all processors should
+        // naturally get mapped to the preferred without thrashing. However,
+        // we can get here if preferred location was set after processors
+        // started thrashing.
+        //
+        // TODO: Bug 2527408. Reset thrashing history when a user policy
+        //       changes in a VA block.
+        hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
+        hint.pin.residency = va_range->preferred_location;
+    }
+    else if (!preferred_location_is_thrashing(va_range, page_thrashing) &&
+             thrashing_processors_have_fast_access_to(va_space, page_thrashing, closest_resident_id)) {
+        // This is a fast path for those scenarios in which all thrashing
+        // processors have fast (NVLINK + native atomics) access to the current
+        // residency. This is skipped if the preferred location is thrashing and
+        // not accessible by the rest of thrashing processors. Otherwise, we
+        // would be in the condition above.
+        if (UVM_ID_IS_CPU(closest_resident_id)) {
+            // On P9 systems, we prefer the CPU to map vidmem (since it can
+            // cache it), so don't map the GPU to sysmem.
+            if (UVM_ID_IS_GPU(requester)) {
+                hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
+                hint.pin.residency = requester;
+            }
+        }
+        else {
+            hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
+            hint.pin.residency = closest_resident_id;
+        }
+    }
+    else if (uvm_id_equal(requester, va_range->preferred_location)) {
+        if (page_thrashing->pinned) {
+            // If the faulting processor is the preferred location, we can
+            // only:
+            // 1) Pin to the preferred location
+            // 2) Throttle if it's pinned elsewhere and we are not the
+            //    do_not_throttle_processor
+            if (uvm_id_equal(va_range->preferred_location, pinned_residency) ||
+                uvm_id_equal(va_range->preferred_location, do_not_throttle_processor)) {
+                hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
+                hint.pin.residency = va_range->preferred_location;
+            }
+            else {
+                hint.type = UVM_PERF_THRASHING_HINT_TYPE_THROTTLE;
+            }
+        }
+        else if (!uvm_id_equal(va_range->preferred_location, do_not_throttle_processor)) {
+            hint.type = UVM_PERF_THRASHING_HINT_TYPE_THROTTLE;
+        }
+        else if (page_thrashing->throttling_count >= va_space_thrashing->params.pin_threshold) {
             hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
             hint.pin.residency = va_range->preferred_location;
         }
-        else if (page_thrashing->pinned && requester == page_thrashing->do_not_throttle_processor) {
+    }
+    else if (page_thrashing->pinned) {
+        // 1) If the requester is the do_not_throttle_processor
+        //    1.1) Pin it to the requester if the rest of processors can access it
+        //    1.2) Pin it to a common location otherwise
+        // 2) Try to map the current pinned residency
+        // 3) Throttle
+        if (uvm_id_equal(requester, do_not_throttle_processor)) {
             hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
 
-            if (closest_resident_id != UVM_MAX_PROCESSORS &&
-                uvm_processor_mask_test(&va_space->accessible_from[closest_resident_id], requester))
-                hint.pin.residency = closest_resident_id;
-            else
+            if (thrashing_processors_can_access(va_space, page_thrashing, requester)) {
                 hint.pin.residency = requester;
-        }
-        else if (requester != page_thrashing->do_not_throttle_processor) {
-            hint.type = UVM_PERF_THRASHING_HINT_TYPE_THROTTLE;
-        }
-
-        return hint;
-    }
-
-    if (closest_resident_id != UVM_MAX_PROCESSORS) {
-        uvm_processor_mask_t fast_to_residency;
-
-        // Combine NVLINK and native atomics mask since we could have PCIe
-        // atomics in the future
-        uvm_processor_mask_and(&fast_to_residency,
-                               &va_space->has_nvlink[closest_resident_id],
-                               &va_space->has_native_atomics[closest_resident_id]);
-        uvm_processor_mask_set(&fast_to_residency, closest_resident_id);
-
-        if (uvm_processor_mask_subset(&page_thrashing->processors, &fast_to_residency)) {
-            if (closest_resident_id == UVM_CPU_ID) {
-                // On P9 systems, we prefer the CPU to map vidmem (since it can
-                // cache it), so don't map the GPU to sysmem.
-                if (requester != UVM_CPU_ID) {
-                    hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
-                    hint.pin.residency = requester;
-                }
             }
             else {
-                hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
-                hint.pin.residency = closest_resident_id;
+                uvm_processor_mask_t common_locations;
+
+                // Find the common location that is closest to the requester
+                thrashing_processors_common_locations(va_space, page_thrashing, &common_locations);
+
+                hint.pin.residency = uvm_processor_mask_find_closest_id(va_space, &common_locations, requester);
+                UVM_ASSERT(UVM_ID_IS_VALID(hint.pin.residency));
             }
-
-            return hint;
         }
-    }
+        else if (uvm_processor_mask_test(&va_space->accessible_from[page_thrashing->pinned_residency_idx], requester)) {
+            UVM_ASSERT(uvm_id_equal(closest_resident_id, pinned_residency));
 
-    if (page_thrashing->pinned ||
-        page_thrashing->throttling_count >= va_space_thrashing->params.pin_threshold) {
-        hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
-
-        if (page_thrashing->pinned &&
-            uvm_processor_mask_test(&va_space->accessible_from[page_thrashing->pinned_residency], requester)) {
-            hint.pin.residency = page_thrashing->pinned_residency;
-        }
-        else if (closest_resident_id != UVM_MAX_PROCESSORS &&
-                 uvm_processor_mask_test(&va_space->accessible_from[closest_resident_id], requester)) {
-            hint.pin.residency = closest_resident_id;
+            hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
+            hint.pin.residency = pinned_residency;
         }
         else {
-            hint.pin.residency = UVM_CPU_ID;
+            hint.type = UVM_PERF_THRASHING_HINT_TYPE_THROTTLE;
         }
     }
-    else if (requester != page_thrashing->do_not_throttle_processor) {
+    else if (!uvm_id_equal(requester, do_not_throttle_processor)) {
         hint.type = UVM_PERF_THRASHING_HINT_TYPE_THROTTLE;
     }
+    else if (page_thrashing->throttling_count >= va_space_thrashing->params.pin_threshold) {
+        hint.type = UVM_PERF_THRASHING_HINT_TYPE_PIN;
+        hint.pin.residency = requester;
+    }
+
+    if (hint.type == UVM_PERF_THRASHING_HINT_TYPE_PIN &&
+        !uvm_va_space_processor_has_memory(va_space, hint.pin.residency))
+        hint.pin.residency = UVM_ID_CPU;
 
     return hint;
 }
@@ -1520,7 +1656,7 @@ uvm_perf_thrashing_hint_t uvm_perf_thrashing_get_hint(uvm_va_block_t *va_block,
         uvm_kvfree(block_thrashing->pages);
         block_thrashing->pages                     = NULL;
         block_thrashing->num_thrashing_pages       = 0;
-        block_thrashing->last_processor            = UVM_MAX_PROCESSORS;
+        block_thrashing->last_processor            = UVM_ID_INVALID;
         block_thrashing->last_time_stamp           = 0;
         block_thrashing->last_thrashing_time_stamp = 0;
         uvm_page_mask_zero(&block_thrashing->thrashing_pages);
@@ -1537,7 +1673,8 @@ uvm_perf_thrashing_hint_t uvm_perf_thrashing_get_hint(uvm_va_block_t *va_block,
     // stamp
     if (uvm_processor_mask_test(&page_thrashing->throttled_processors, requester)) {
         NvU64 throttling_end_time_stamp = page_thrashing_get_throttling_end_time_stamp(page_thrashing);
-        if (time_stamp < throttling_end_time_stamp && requester != page_thrashing->do_not_throttle_processor) {
+        if (time_stamp < throttling_end_time_stamp &&
+            !uvm_id_equal(requester, uvm_id(page_thrashing->do_not_throttle_processor_idx))) {
             hint.type = UVM_PERF_THRASHING_HINT_TYPE_THROTTLE;
             goto done;
         }
@@ -1564,8 +1701,9 @@ uvm_perf_thrashing_hint_t uvm_perf_thrashing_get_hint(uvm_va_block_t *va_block,
     // Update throttling heuristics
     thrashing_throttle_update(va_space_thrashing, va_block, page_thrashing, requester, time_stamp);
 
-    if (page_thrashing->pinned && page_thrashing->has_revocation_events &&
-        requester != page_thrashing->do_not_throttle_processor) {
+    if (page_thrashing->pinned &&
+        page_thrashing->has_revocation_events &&
+        !uvm_id_equal(requester, uvm_id(page_thrashing->do_not_throttle_processor_idx))) {
 
         // When we get revocation thrashing, this is due to system-wide atomics
         // downgrading the permissions of other processors. Revocations only
@@ -1599,10 +1737,10 @@ done:
             hint.type = UVM_PERF_THRASHING_HINT_TYPE_THROTTLE;
         }
         else {
-            if (hint.pin.residency == requester)
-                PROCESSOR_THRASHING_STATS_INC(hint.pin.residency, num_pin_local);
+            if (uvm_id_equal(hint.pin.residency, requester))
+                PROCESSOR_THRASHING_STATS_INC(va_space, requester, num_pin_local);
             else
-                PROCESSOR_THRASHING_STATS_INC(hint.pin.residency, num_pin_remote);
+                PROCESSOR_THRASHING_STATS_INC(va_space, requester, num_pin_remote);
 
             uvm_processor_mask_copy(&hint.pin.processors, &page_thrashing->processors);
         }
@@ -1615,14 +1753,14 @@ done:
                                      page_index,
                                      requester);
 
-        PROCESSOR_THRASHING_STATS_INC(requester, num_throttle);
+        PROCESSOR_THRASHING_STATS_INC(va_space, requester, num_throttle);
 
         hint.throttle.end_time_stamp = page_thrashing_get_throttling_end_time_stamp(page_thrashing);
     }
     else if (hint.type == UVM_PERF_THRASHING_HINT_TYPE_NONE && page_thrashing) {
         UVM_ASSERT(!uvm_processor_mask_test(&page_thrashing->throttled_processors, requester));
         UVM_ASSERT(!page_thrashing->pinned);
-        UVM_ASSERT(page_thrashing->pinned_residency == UVM_MAX_PROCESSORS);
+        UVM_ASSERT(UVM_ID_IS_INVALID(uvm_id(page_thrashing->pinned_residency_idx)));
     }
 
     return hint;
@@ -1762,6 +1900,11 @@ exit_no_list_lock:
     uvm_va_space_up_read(va_space_thrashing->va_space);
 }
 
+static void thrashing_unpin_pages_entry(struct work_struct *work)
+{
+    UVM_ENTRY_VOID(thrashing_unpin_pages(work));
+}
+
 NV_STATUS uvm_perf_thrashing_load(uvm_va_space_t *va_space)
 {
     va_space_thrashing_info_t *va_space_thrashing;
@@ -1777,7 +1920,7 @@ NV_STATUS uvm_perf_thrashing_load(uvm_va_space_t *va_space)
 
     uvm_spin_lock_init(&va_space_thrashing->pinned_pages.lock, UVM_LOCK_ORDER_LEAF);
     INIT_LIST_HEAD(&va_space_thrashing->pinned_pages.list);
-    INIT_DELAYED_WORK(&va_space_thrashing->pinned_pages.dwork, thrashing_unpin_pages);
+    INIT_DELAYED_WORK(&va_space_thrashing->pinned_pages.dwork, thrashing_unpin_pages_entry);
 
     return NV_OK;
 }
@@ -1818,6 +1961,20 @@ void uvm_perf_thrashing_unload(uvm_va_space_t *va_space)
     }
 }
 
+NV_STATUS uvm_perf_thrashing_register_gpu(uvm_va_space_t *va_space, uvm_gpu_t *gpu)
+{
+    // If a simulated GPU is registered, re-initialize thrashing parameters in
+    // case they need to be adjusted
+    if (g_uvm_global.num_simulated_devices > 0) {
+        va_space_thrashing_info_t *va_space_thrashing = va_space_thrashing_info_get(va_space);
+
+        if (!va_space_thrashing->params.test_overrides)
+            va_space_thrashing_info_init_params(va_space_thrashing);
+    }
+
+    return NV_OK;
+}
+
 NV_STATUS uvm_perf_thrashing_init()
 {
     NV_STATUS status;
@@ -1937,6 +2094,7 @@ NV_STATUS uvm8_test_set_page_thrashing_policy(UVM_TEST_SET_PAGE_THRASHING_POLICY
     uvm_va_space_down_write(va_space);
 
     va_space_thrashing = va_space_thrashing_info_get(va_space);
+    va_space_thrashing->params.test_overrides = true;
 
     if (params->policy == UVM_TEST_PAGE_THRASHING_POLICY_ENABLE) {
         if (va_space_thrashing->params.enable)
