@@ -173,6 +173,12 @@ static NV_STATUS chunk_alloc_check(uvm_pmm_gpu_t *pmm,
     NV_STATUS status;
     uvm_tracker_t local_tracker = UVM_TRACKER_INIT();
 
+    // If the GPU has no memory then PMA will fail with NV_ERR_INVALID_ARGUMENT.
+    // Callers assert that only NV_OK or NV_ERR_NO_MEMORY are returned here, so
+    // replace the error to avoid noise.
+    if (pmm->gpu->mem_info.size == 0)
+        return NV_ERR_NO_MEMORY;
+
     status = uvm_pmm_gpu_alloc(pmm, num_chunks, chunk_size, mem_type, flags, chunks, &local_tracker);
     if (status != NV_OK)
         return status;
@@ -840,6 +846,7 @@ NV_STATUS uvm8_test_pmm_async_alloc(UVM_TEST_PMM_ASYNC_ALLOC_PARAMS *params, str
     uvm_tracker_t tracker = UVM_TRACKER_INIT();
     uvm_push_t push;
     NvU32 i;
+    uvm_global_processor_mask_t global_mask;
 
     chunks = uvm_kvmalloc_zero(params->num_chunks * sizeof(chunks[0]));
     if (!chunks)
@@ -859,7 +866,10 @@ NV_STATUS uvm8_test_pmm_async_alloc(UVM_TEST_PMM_ASYNC_ALLOC_PARAMS *params, str
     status = uvm_mem_alloc(&mem_params, &dummy_buffer);
     if (status != NV_OK)
         goto out;
-    status = uvm_mem_map_kernel(dummy_buffer, &va_space->registered_gpus);
+
+    uvm_va_space_global_gpus(va_space, &global_mask);
+
+    status = uvm_mem_map_kernel(dummy_buffer, &global_mask);
     if (status != NV_OK)
         goto out;
 
@@ -988,7 +998,7 @@ static NV_STATUS test_pmm_reverse_map_many_blocks(uvm_gpu_t *gpu, uvm_va_space_t
 
     uvm_mutex_lock(&va_block->lock);
 
-    is_resident = (uvm_va_block_page_get_closest_resident(va_block, 0, gpu->id) == gpu->id);
+    is_resident = uvm_id_equal(uvm_va_block_page_get_closest_resident(va_block, 0, gpu->id), gpu->id);
     if (is_resident) {
         phys_addr = uvm_va_block_gpu_phys_page_address(va_block, 0, gpu);
         phys_addr.address = UVM_ALIGN_DOWN(phys_addr.address, UVM_VA_BLOCK_SIZE);
@@ -1178,7 +1188,7 @@ NV_STATUS uvm8_test_pmm_indirect_peers(UVM_TEST_PMM_INDIRECT_PEERS_PARAMS *param
     uvm_va_space_down_read(va_space);
 
     for_each_va_space_gpu(owning_gpu, va_space) {
-        for_each_gpu_in_mask(accessing_gpu, &va_space->indirect_peers[owning_gpu->id]) {
+        for_each_va_space_gpu_in_mask(accessing_gpu, va_space, &va_space->indirect_peers[uvm_id_value(owning_gpu->id)]) {
             ran_test = true;
             status = test_indirect_peers(owning_gpu, accessing_gpu);
             if (status != NV_OK)
