@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2017 NVIDIA Corporation
+    Copyright (c) 2017-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -39,7 +39,7 @@ void uvm_hal_turing_arch_init_properties(uvm_gpu_t *gpu)
 
     gpu->utlb_per_gpc_count = uvm_turing_get_utlbs_per_gpc(gpu);
 
-    gpu->fault_buffer_info.replayable.utlb_count = gpu->gpc_count * gpu->utlb_per_gpc_count;
+    gpu->fault_buffer_info.replayable.utlb_count = gpu->rm_info.gpcCount * gpu->utlb_per_gpc_count;
     {
         uvm_fault_buffer_entry_t *dummy;
         UVM_ASSERT(gpu->fault_buffer_info.replayable.utlb_count <= (1 << (sizeof(dummy->fault_source.utlb_id) * 8)));
