@@ -28,17 +28,17 @@
 // This is really a struct UvmOpsUvmEvents *. It needs to be an atomic because
 // it can be read outside of the g_pNvUvmEventsLock. Use getUvmEvents and
 // setUvmEvents to access it.
-static nv_atomic_long_t g_pNvUvmEvents;
+static atomic_long_t g_pNvUvmEvents;
 static struct semaphore g_pNvUvmEventsLock;
 
 static struct UvmOpsUvmEvents *getUvmEvents(void)
 {
-    return (struct UvmOpsUvmEvents *)NV_ATOMIC_LONG_READ(g_pNvUvmEvents);
+    return (struct UvmOpsUvmEvents *)atomic_long_read(&g_pNvUvmEvents);
 }
 
 static void setUvmEvents(struct UvmOpsUvmEvents *newEvents)
 {
-    NV_ATOMIC_LONG_SET(g_pNvUvmEvents, (long)newEvents);
+    atomic_long_set(&g_pNvUvmEvents, (long)newEvents);
 }
 
 static nvidia_stack_t *g_sp;
@@ -202,8 +202,39 @@ NV_STATUS nvUvmInterfaceSessionDestroy(uvmGpuSessionHandle session)
 }
 EXPORT_SYMBOL(nvUvmInterfaceSessionDestroy);
 
-NV_STATUS nvUvmInterfaceDupAddressSpace(uvmGpuSessionHandle session,
-                                        const NvProcessorUuid *gpuUuid,
+NV_STATUS nvUvmInterfaceDeviceCreate(uvmGpuSessionHandle session,
+                                     const NvProcessorUuid *gpuUuid,
+                                     uvmGpuDeviceHandle *device)
+{
+    nvidia_stack_t *sp = NULL;
+    NV_STATUS status;
+
+    if (nv_kmem_cache_alloc_stack(&sp) != 0)
+    {
+        return NV_ERR_NO_MEMORY;
+    }
+
+    status = rm_gpu_ops_device_create(sp,
+                                      (gpuSessionHandle)session,
+                                      gpuUuid,
+                                      (gpuDeviceHandle *)device);
+
+    nv_kmem_cache_free_stack(sp);
+    return status;
+}
+EXPORT_SYMBOL(nvUvmInterfaceDeviceCreate);
+
+void nvUvmInterfaceDeviceDestroy(uvmGpuDeviceHandle device)
+{
+    nvidia_stack_t *sp = nvUvmGetSafeStack();
+
+    rm_gpu_ops_device_destroy(sp, (gpuDeviceHandle)device);
+
+    nvUvmFreeSafeStack(sp);
+}
+EXPORT_SYMBOL(nvUvmInterfaceDeviceDestroy);
+
+NV_STATUS nvUvmInterfaceDupAddressSpace(uvmGpuDeviceHandle device,
                                         NvHandle hUserClient,
                                         NvHandle hUserVASpace,
                                         uvmGpuAddressSpaceHandle *vaSpace,
@@ -218,8 +249,7 @@ NV_STATUS nvUvmInterfaceDupAddressSpace(uvmGpuSessionHandle session,
     }
 
     status = rm_gpu_ops_dup_address_space(sp,
-                                          (gpuSessionHandle)session,
-                                          gpuUuid,
+                                          (gpuDeviceHandle)device,
                                           hUserClient,
                                           hUserVASpace,
                                           (gpuAddressSpaceHandle *)vaSpace,
@@ -230,8 +260,7 @@ NV_STATUS nvUvmInterfaceDupAddressSpace(uvmGpuSessionHandle session,
 }
 EXPORT_SYMBOL(nvUvmInterfaceDupAddressSpace);
 
-NV_STATUS nvUvmInterfaceAddressSpaceCreate(uvmGpuSessionHandle session,
-                                           const NvProcessorUuid *gpuUuid,
+NV_STATUS nvUvmInterfaceAddressSpaceCreate(uvmGpuDeviceHandle device,
                                            unsigned long long vaBase,
                                            unsigned long long vaSize,
                                            uvmGpuAddressSpaceHandle *vaSpace,
@@ -246,8 +275,7 @@ NV_STATUS nvUvmInterfaceAddressSpaceCreate(uvmGpuSessionHandle session,
     }
 
     status = rm_gpu_ops_address_space_create(sp,
-                                             (gpuSessionHandle)session,
-                                             gpuUuid,
+                                             (gpuDeviceHandle)device,
                                              vaBase,
                                              vaSize,
                                              (gpuAddressSpaceHandle *)vaSpace,
@@ -336,7 +364,9 @@ NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
 
 EXPORT_SYMBOL(nvUvmInterfaceGetP2PCaps);
 
-NV_STATUS nvUvmInterfaceGetPmaObject(const NvProcessorUuid *gpuUUID, void **pPma)
+NV_STATUS nvUvmInterfaceGetPmaObject(uvmGpuAddressSpaceHandle vaSpace,
+                                     void **pPma,
+                                     const UvmPmaStatistics **pPmaPubStats)
 {
     nvidia_stack_t *sp = NULL;
     NV_STATUS status;
@@ -346,7 +376,7 @@ NV_STATUS nvUvmInterfaceGetPmaObject(const NvProcessorUuid *gpuUUID, void **pPma
         return NV_ERR_NO_MEMORY;
     }
 
-    status = rm_gpu_ops_get_pma_object(sp, gpuUUID, pPma);
+    status = rm_gpu_ops_get_pma_object(sp, (gpuAddressSpaceHandle)vaSpace, pPma, (const nvgpuPmaStatistics_t *)pPmaPubStats);
 
     nv_kmem_cache_free_stack(sp);
     return status;
@@ -594,7 +624,7 @@ NV_STATUS nvUvmInterfaceGetGpuInfo(const NvProcessorUuid *gpuUuid, UvmGpuInfo *p
 }
 EXPORT_SYMBOL(nvUvmInterfaceGetGpuInfo);
 
-NV_STATUS nvUvmInterfaceServiceDeviceInterruptsRM(uvmGpuAddressSpaceHandle vaSpace)
+NV_STATUS nvUvmInterfaceServiceDeviceInterruptsRM(uvmGpuDeviceHandle device)
 {
     nvidia_stack_t *sp = NULL;
     NV_STATUS status;
@@ -605,7 +635,7 @@ NV_STATUS nvUvmInterfaceServiceDeviceInterruptsRM(uvmGpuAddressSpaceHandle vaSpa
     }
 
     status = rm_gpu_ops_service_device_interrupts_rm(sp,
-                                                    (gpuAddressSpaceHandle) vaSpace);
+                                                    (gpuDeviceHandle)device);
 
     nv_kmem_cache_free_stack(sp);
     return status;
@@ -733,7 +763,7 @@ NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuAddressSpaceHandle vaSpace,
 }
 EXPORT_SYMBOL(nvUvmInterfaceGetFbInfo);
 
-NV_STATUS nvUvmInterfaceOwnPageFaultIntr(const NvProcessorUuid *gpuUuid, NvBool bOwnInterrupts)
+NV_STATUS nvUvmInterfaceOwnPageFaultIntr(uvmGpuDeviceHandle device, NvBool bOwnInterrupts)
 {
     nvidia_stack_t *sp = NULL;
     NV_STATUS status;
@@ -743,15 +773,15 @@ NV_STATUS nvUvmInterfaceOwnPageFaultIntr(const NvProcessorUuid *gpuUuid, NvBool
         return NV_ERR_NO_MEMORY;
     }
 
-    status = rm_gpu_ops_own_page_fault_intr(sp, gpuUuid, bOwnInterrupts);
+    status = rm_gpu_ops_own_page_fault_intr(sp, (gpuDeviceHandle)device, bOwnInterrupts);
     nv_kmem_cache_free_stack(sp);
     return status;
 }
 EXPORT_SYMBOL(nvUvmInterfaceOwnPageFaultIntr);
 
 
-NV_STATUS nvUvmInterfaceInitFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuFaultInfo *pFaultInfo)
+NV_STATUS nvUvmInterfaceInitFaultInfo(uvmGpuDeviceHandle device,
+                                      UvmGpuFaultInfo *pFaultInfo)
 {
     nvidia_stack_t *sp = NULL;
     NV_STATUS status;
@@ -762,7 +792,7 @@ NV_STATUS nvUvmInterfaceInitFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
     }
 
     status = rm_gpu_ops_init_fault_info(sp,
-                                       (gpuAddressSpaceHandle)vaSpace,
+                                       (gpuDeviceHandle)device,
                                        pFaultInfo);
 
     // Preallocate a stack for functions called from ISR top half
@@ -787,7 +817,7 @@ NV_STATUS nvUvmInterfaceInitFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
         if (err)
         {
             rm_gpu_ops_destroy_fault_info(sp,
-                                          (gpuAddressSpaceHandle)vaSpace,
+                                          (gpuDeviceHandle)device,
                                           pFaultInfo);
 
             status = NV_ERR_NO_MEMORY;
@@ -799,8 +829,8 @@ NV_STATUS nvUvmInterfaceInitFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
 }
 EXPORT_SYMBOL(nvUvmInterfaceInitFaultInfo);
 
-NV_STATUS nvUvmInterfaceInitAccessCntrInfo(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuAccessCntrInfo *pAccessCntrInfo)
+NV_STATUS nvUvmInterfaceInitAccessCntrInfo(uvmGpuDeviceHandle device,
+                                           UvmGpuAccessCntrInfo *pAccessCntrInfo)
 {
     nvidia_stack_t *sp = NULL;
     NV_STATUS status;
@@ -811,17 +841,17 @@ NV_STATUS nvUvmInterfaceInitAccessCntrInfo(uvmGpuAddressSpaceHandle vaSpace,
     }
 
     status = rm_gpu_ops_init_access_cntr_info(sp,
-                                       (gpuAddressSpaceHandle)vaSpace,
-                                       pAccessCntrInfo);
+                                              (gpuDeviceHandle)device,
+                                              pAccessCntrInfo);
 
     nv_kmem_cache_free_stack(sp);
     return status;
 }
 EXPORT_SYMBOL(nvUvmInterfaceInitAccessCntrInfo);
 
-NV_STATUS nvUvmInterfaceEnableAccessCntr(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuAccessCntrInfo *pAccessCntrInfo,
-    UvmGpuAccessCntrConfig *pAccessCntrConfig)
+NV_STATUS nvUvmInterfaceEnableAccessCntr(uvmGpuDeviceHandle device,
+                                         UvmGpuAccessCntrInfo *pAccessCntrInfo,
+                                         UvmGpuAccessCntrConfig *pAccessCntrConfig)
 {
     nvidia_stack_t *sp = NULL;
     NV_STATUS status;
@@ -832,17 +862,17 @@ NV_STATUS nvUvmInterfaceEnableAccessCntr(uvmGpuAddressSpaceHandle vaSpace,
     }
 
     status = rm_gpu_ops_enable_access_cntr (sp,
-                                       (gpuAddressSpaceHandle)vaSpace,
-                                       pAccessCntrInfo,
-                                       pAccessCntrConfig);
+                                            (gpuDeviceHandle)device,
+                                            pAccessCntrInfo,
+                                            pAccessCntrConfig);
 
     nv_kmem_cache_free_stack(sp);
     return status;
 }
 EXPORT_SYMBOL(nvUvmInterfaceEnableAccessCntr);
 
-NV_STATUS nvUvmInterfaceDestroyFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuFaultInfo *pFaultInfo)
+NV_STATUS nvUvmInterfaceDestroyFaultInfo(uvmGpuDeviceHandle device,
+                                         UvmGpuFaultInfo *pFaultInfo)
 {
     nvidia_stack_t *sp = nvUvmGetSafeStack();
     NV_STATUS status;
@@ -861,7 +891,7 @@ NV_STATUS nvUvmInterfaceDestroyFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
     }
 
     status = rm_gpu_ops_destroy_fault_info(sp,
-                                          (gpuAddressSpaceHandle)vaSpace,
+                                          (gpuDeviceHandle)device,
                                           pFaultInfo);
 
     nvUvmFreeSafeStack(sp);
@@ -889,30 +919,30 @@ NV_STATUS nvUvmInterfaceGetNonReplayableFaults(UvmGpuFaultInfo *pFaultInfo,
 }
 EXPORT_SYMBOL(nvUvmInterfaceGetNonReplayableFaults);
 
-NV_STATUS nvUvmInterfaceDestroyAccessCntrInfo(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuAccessCntrInfo *pAccessCntrInfo)
+NV_STATUS nvUvmInterfaceDestroyAccessCntrInfo(uvmGpuDeviceHandle device,
+                                              UvmGpuAccessCntrInfo *pAccessCntrInfo)
 {
     nvidia_stack_t *sp = nvUvmGetSafeStack();
     NV_STATUS status;
 
     status = rm_gpu_ops_destroy_access_cntr_info(sp,
-                                          (gpuAddressSpaceHandle)vaSpace,
-                                          pAccessCntrInfo);
+                                                 (gpuDeviceHandle)device,
+                                                 pAccessCntrInfo);
 
     nvUvmFreeSafeStack(sp);
     return status;
 }
 EXPORT_SYMBOL(nvUvmInterfaceDestroyAccessCntrInfo);
 
-NV_STATUS nvUvmInterfaceDisableAccessCntr(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuAccessCntrInfo *pAccessCntrInfo)
+NV_STATUS nvUvmInterfaceDisableAccessCntr(uvmGpuDeviceHandle device,
+                                          UvmGpuAccessCntrInfo *pAccessCntrInfo)
 {
     nvidia_stack_t *sp = nvUvmGetSafeStack();
     NV_STATUS status;
 
     status = rm_gpu_ops_disable_access_cntr(sp,
-                                          (gpuAddressSpaceHandle)vaSpace,
-                                          pAccessCntrInfo);
+                                            (gpuDeviceHandle)device,
+                                            pAccessCntrInfo);
 
     nvUvmFreeSafeStack(sp);
     return status;
